package org.hathitrust;

import java.io.Serializable;
import org.apache.commons.cli.*;

import org.apache.spark.api.java.*;
import org.apache.spark.SparkConf;

public class PrepareForIngest implements Serializable
{
	private static final long serialVersionUID = 1L;

	public static final int NUM_PARTITIONS = 6; // default would appear to be 2
	
	protected String _input_dir;
	protected String _json_list_filename;
	protected String _output_dir;
	
	protected int    _verbosity;

	public PrepareForIngest(String input_dir, String json_list_filename, String output_dir, int verbosity)
	{
		_input_dir = input_dir;
		_json_list_filename = (json_list_filename != null) ? json_list_filename : input_dir;
		_output_dir = output_dir;
		_verbosity = verbosity;
	}

	public void exec()
	{	
		String spark_app_name = "HathiTrust Extract Features: Prepare for Solr Ingest";
		spark_app_name += " [" + _json_list_filename + "]";

		SparkConf conf = new SparkConf().setAppName(spark_app_name);
		JavaSparkContext jsc = new JavaSparkContext(conf);
		
		if (_verbosity >= 2) {
			System.out.println("Default Minimum Partions: " + jsc.defaultMinPartitions());
			System.out.println("Default Parallelism: " + jsc.defaultParallelism());
		}
				
		JavaRDD<String> json_list_data = jsc.textFile(_json_list_filename,NUM_PARTITIONS).cache();

		PagedJSON paged_json = new PagedJSON(_input_dir,_output_dir,_verbosity);
		JavaRDD<String> json_ids = json_list_data.flatMap(paged_json).cache();

		json_ids.saveAsTextFile("foo");

		long num_ids = json_ids.count();
		System.out.println("");
		System.out.println("############");
		System.out.println("# Number of page ids: " + num_ids);
		System.out.println("############");
		System.out.println("");

		jsc.close();
	}

	public static void main(String[] args) {


		Options options = new Options();

		//.withType(Integer.class)
/*
		options.addOption(OptionBuilder.withLongOpt("verbosity")
				.withDescription("Set to control the level of debugging output [0=none, 1=some, 2=lots]")
				.hasArg()
				.withArgName("v")
				.isRequired(false)
				.create());
*/
		//Option num_cores_opt = new Option("n", "num-cores", true, "Number of cores to use");
		//num_cores_opt.setRequired(false);
		//options.addOption(num_cores_opt);

		Option verbosity_opt = new Option("v", "verbosity", true, 
				"Set to control the level of debugging output [0=none, 1=some, 2=lots]");
		verbosity_opt.setRequired(false);
		options.addOption(verbosity_opt);
		
		//CommandLineParser parser = new DefaultParser(); // 1.3 and above
		
		// need to work with CLI v1.2 as this is the JAR that is bundled with Hadoop/Spark 
		CommandLineParser parser = new GnuParser(); 
		HelpFormatter formatter = new HelpFormatter();
		CommandLine cmd;

		try {
			cmd = parser.parse(options, args);
		}
		catch (ParseException e) {
			System.err.println(e.getMessage());
			formatter.printHelp("RUN.bash [options] json-file-list.txt input-dir output-dir", options);
			System.exit(1);
			return;
		}

		//value = ((Integer)cmdLine.getParsedOptionValue("num-cores")).intValue();
		//value = ((Integer)cmdLine.getOptionValue("num-cores","2")).intValue();

		//cmd.hasOption("json-filelist")
		
		String verbosity_str = cmd.getOptionValue("verbosity","0");
		int verbosity = Integer.parseInt(verbosity_str);

		String[] filtered_args = cmd.getArgs();

		if (filtered_args.length != 3) {
			formatter.printHelp("RUN.bash [options] json-filelist.txt input-dir output-dir", options);
			System.exit(1);
		}
		String json_list_filename = filtered_args[0];
		String input_dir  = filtered_args[1];
		String output_dir = filtered_args[2];

		PrepareForIngest prep_for_ingest = new PrepareForIngest(input_dir,json_list_filename,output_dir,verbosity);
		prep_for_ingest.exec();

	}
}
